angular.module('starter.controllers', [])

.controller('NearbyCtrl', function ($scope, $rootScope, $cordovaToast, DistanceTools, DateTimeTools, LocationTools) {
    // Check if current position available - if not display error/alternative view with all points
    $scope.nearbyHotspots = $scope.$storage.wifispots;
    $scope.findingLocation = true;
    $scope.oldLocation = false;

    $scope.$on('UpdateHotspots', function (event, args) {
        console.log('hotspot update');
        $scope.nearbyHotspots = $scope.$storage.wifispots;
        recalculateNearbySpots();       
    });

    $scope.$on('CurrentPosition', function (event, args) {
        if (args.type == 'location') {
            console.log('NearbyCtrl - CurrentPosition broadcast');
            recalculateNearbySpots();
            $scope.findingLocation = false;
            $scope.oldLocation = false;
        } else if (args.type == 'fail') {
            $cordovaToast.showLongBottom('GPS Positioning failed, please ensure GPS is enabled and click Refresh');
        }
    });

    $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
        if (toState.name == 'tab.nearby') {
            // in case hotspots have just been updated
            recalculateNearbySpots();

            if ($rootScope.currentPosition) {
                $scope.findingLocation = false;

                if (DateTimeTools.getTimeDifferenceSeconds($rootScope.currentPositionTimestamp) > 60)
                    $scope.oldLocation = true;
            }
        }
    });

    function recalculateNearbySpots() {
        $scope.nearbyHotspots = [];
        $scope.filteredSpots = [];

        // Build a rectangle and only return hotspots within that box to speed up the query
        // 1 of latitude ~= 69 miles, 1 of longitude ~= cos(latitude)*69 
        // 2km = 1.24274m, 4km = 2.48548, 7km = 4.3496
        if ($rootScope.currentPosition) $scope.filteredSpots = DistanceTools.filterWithinRectangle($scope.$storage.wifispots, $rootScope.currentPosition.coords, 1.24274);
        if ($scope.filteredSpots.length < 10) $scope.filteredSpots = $scope.filteredSpots = DistanceTools.filterWithinRectangle($scope.$storage.wifispots, $rootScope.currentPosition.coords, 2.48548);
        if ($scope.filteredSpots.length < 10) $scope.filteredSpots = $scope.filteredSpots = DistanceTools.filterWithinRectangle($scope.$storage.wifispots, $rootScope.currentPosition.coords, 4.3496);
        if ($scope.filteredSpots.length == 0) $scope.filteredSpots = $scope.$storage.wifispots;
        
        angular.forEach($scope.filteredSpots, function (spot) {
            if ($rootScope.currentPosition) {
                spot.distance = DistanceTools.getDistance($rootScope.currentPosition.coords.latitude, $rootScope.currentPosition.coords.longitude, spot.latitude, spot.longitude);

                spot.distanceText = DistanceTools.formatDistance(spot.distance);
            }
            this.push(spot);
        }, $scope.nearbyHotspots);

        $scope.locationExists = $rootScope.currentPosition;
    };

    $scope.refreshNearby = function () {
        console.log("Refresh Nearby Status");
        $cordovaToast.showLongBottom('Refreshing GPS status');
        LocationTools.stopWatch();
        LocationTools.startWatch();
    };
})

.controller('MapCtrl', function ($scope, $rootScope, $location, $cordovaToast, NetworkTools, DistanceTools, DateTimeTools, LocationTools) {
    console.log('MapCtrl general');
    var geoMarker;
    $scope.findingLocation = true;
    $scope.oldLocation = false;

    if (!NetworkTools.isOnline())
        $cordovaToast.showLongBottom('A data connection is required to display the map. Enable data and restart app.');
    
    function setMarkers() {
        var infoWindow = new google.maps.InfoWindow();
        var createMarker = function (info) {
            var marker = new google.maps.Marker({
                position: new google.maps.LatLng(info.latitude, info.longitude),
                map: $scope.map,
                animation: google.maps.Animation.DROP,
                title: info.title + ', ' + info.city,
                icon: 'img/20x32.png'
            });

            marker.content = '<div class="marker-info-window">' + info.title + '</div>';
            marker.spotId = info.id;
            marker.infowindow = infoWindow;

            google.maps.event.addListener(marker, 'click', function () {
                distance = getDistanceToMarker(info.latitude, info.longitude);
                if (distance != '')
                    distance = '<span class="marker-info-window-distance">' + distance + '</span>';
                infoWindow.setContent(marker.content + distance);
                infoWindow.open($scope.map, marker);
            });
            if (!$scope.markers) $scope.markers = [];
            $scope.markers.push(marker);
        }

        for (i = 0; i < $scope.$storage.wifispots.length; i++) {
            createMarker($scope.$storage.wifispots[i]);
        }

        if ($rootScope.currentPosition != null)
            showCurrentPosition();

        if ($location.search().s != null)
            highlightSpot();
    };

    $scope.mapCreated = function (map) {
        console.log('mapCreated called');
        $scope.map = map;
        setMarkers();
    };
    
    $scope.$on('UpdateHotspots', function (event, args) {
        console.log('hotspot update');
        if ($scope.markers) {
            angular.forEach($scope.markers, function (marker) {
                marker.setMap(null);
            });
        }
        $scope.markers = [];
        setMarkers();
    });

    $scope.$on('CurrentPosition', function (event, args) {
        if (args.type == 'location') {
            console.log('MapCtrl - CurrentPosition broadcast');
            showCurrentPosition();
            $scope.findingLocation = false;
            $scope.oldLocation = false;
        }
    });

    $scope.centerOnMe = function() {
        console.log("Centering");
        if (!$scope.map) {
            console.log('No map/position');
            return;
        }

        if (!$rootScope.currentPosition) {
            $cordovaToast.showLongBottom('Current location is not available - please ensure GPS is enabled.');
            LocationTools.stopWatch();
            LocationTools.startWatch();
            return;
        }

        $scope.map.panTo(new google.maps.LatLng($rootScope.currentPosition.coords.latitude, $rootScope.currentPosition.coords.longitude));
        $scope.map.setZoom(16);
    };

    $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
        if (toState.name == 'tab.map') {
            clearOpenMarkers();
            highlightSpot();

            if ($rootScope.currentPosition) {
                $scope.findingLocation = false;

                if (DateTimeTools.getTimeDifferenceSeconds($rootScope.currentPositionTimestamp) > 60)
                    $scope.oldLocation = true;
            }
        }
    });

    function getDistanceToMarker(lat,lng) {
        if ($rootScope.currentPosition) {
            distance = DistanceTools.getDistance($rootScope.currentPosition.coords.latitude, $rootScope.currentPosition.coords.longitude, lat,lng);
            return DistanceTools.formatDistance(distance);
        }

        return '';
    };

    function highlightSpot() {
        console.log('highlightSpot - ' + $location.search().s);
        if ($location.search().s != null) {
            if ($scope.markers) {
                angular.forEach($scope.markers, function (marker) {
                    if (marker.spotId == $location.search().s) {
                        $scope.map.panTo(marker.position);
                        $scope.map.setZoom(16);
                        google.maps.event.trigger(marker, 'click');
                    }
                });
            }
        } else {
            showCurrentPosition();
            $scope.centerOnMe();
        }
    };

    function clearOpenMarkers() {
        if ($scope.markers) {
            angular.forEach($scope.markers, function (marker) {
                marker.infowindow.close();
            });
        }
    };

    function showCurrentPosition() {
        console.log('showCurrentPosition');
        if (!geoMarker && $rootScope.currentPosition != null) {
            geoMarker = new google.maps.Marker({
                map: $scope.map,
                icon: 'img/icon_BlueDot.png'
            });

            if (!$location.search().s)
                $scope.centerOnMe();
        }

        geoMarker.setPosition(new google.maps.LatLng($rootScope.currentPosition.coords.latitude, $rootScope.currentPosition.coords.longitude));
    };
})

.controller('AboutCtrl', function ($scope, $ionicPlatform, UpdateTools) {
    $scope.rateApp = function () {
        if ($ionicPlatform.is('ios')) {
            window.open('itms://itunes.apple.com/app/id1078073795?mt=8', '_system');  //itms-apps://itunes.apple.com/us/app/mca-malta-free-wifi/id1078073795?ls=1&mt=8 // or itms://
        } else if ($ionicPlatform.is('android')) {
            window.open('market://details?id=mt.org.mca.maltafreewifi', '_system');
        }
    };

    $scope.checkForUpdates = function () {
        UpdateTools.$updateNow(true, true);
    };

    $scope.totalHotspots = $scope.$storage.wifispots.length;
})

.controller('AboutContactCtrl', function ($scope, $location, $cordovaToast, $http, NetworkTools, EmailTools) {
    $scope.send = function (contactForm) {
        console.log('submitted');
        if (contactForm.$valid) {
            console.log('form valid');

            function rCallback(result) {
                if (result == 'success') {
                    $cordovaToast.showLongBottom('Message successfully sent');
                } else {
                    $cordovaToast.showLongBottom('Failed to send message, stored for later delivery');
                }
            }

            var payload = {
                to: 'mcafreewifi@gmail.com',
                from: contactForm.email.$viewValue,
                subject: 'MCA Free Wi-Fi Feedback - ' + contactForm.category.$viewValue,
                text: contactForm.comments.$viewValue
            };

            if (!NetworkTools.isOnline()) {
                if ($scope.$storage.pendingposts == null) $scope.$storage.pendingposts = [];
                $scope.$storage.pendingposts.push(payload);
                $cordovaToast.showLongBottom('Message will be sent when a data connection is available');
            } else {
                // call mail service
                EmailTools.$send(payload, rCallback, -1);
            }
            $location.path('/tab/about');
        }
    };
})

.controller('SpotDetailCtrl', function ($scope, $rootScope, $stateParams, $cordovaDeviceOrientation, DistanceTools, DateTimeTools) {
    $scope.headingAvailable = false;
    $scope.findingLocation = true;
    $scope.oldLocation = false;

    var getHotspot = function (spotId, wifispots) {
        for (var i = 0; i < wifispots.length; i++) {
            if (wifispots[i].id === parseInt(spotId)) {
                return wifispots[i];
            }
        }
        return null;
    };

    $scope.spot = getHotspot($stateParams.spotId, $scope.$storage.wifispots);

    $scope.$on('CurrentPosition', function (event, args) {
        console.log('SpotDetailCtrl - CurrentPosition broadcast');
        if ($rootScope.currentPosition) {
            $scope.spot.distance = DistanceTools.getDistance($rootScope.currentPosition.coords.latitude, $rootScope.currentPosition.coords.longitude, $scope.spot.latitude, $scope.spot.longitude);
            $scope.spot.distanceText = DistanceTools.formatDistance($scope.spot.distance);
        }

        $scope.nearby = $scope.spot.distance < 20;

        if (!$rootScope.currentHeading) {
            console.log('SpotDetailCtrl - CurrentPosition broadcast (no heading)');
        } else {
            if (args.type == 'compass') {
                $scope.headingAvailable = true;
                updateDirection();
            }
        }

        $scope.findingLocation = false;
        $scope.oldLocation = false;
    });

    $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
        if (toState.name == 'tab.spot-detail') {
            if ($rootScope.currentPosition) {
                $scope.findingLocation = false;

                if (DateTimeTools.getTimeDifferenceSeconds($rootScope.currentPositionTimestamp) > 60)
                    $scope.oldLocation = true;
            }
        }
    });

    function updateDirection() {
        destinationBearing = DistanceTools.getBearing($rootScope.currentPosition.coords.latitude, $rootScope.currentPosition.coords.longitude, $scope.spot.latitude, $scope.spot.longitude);
        //$scope.spot.magneticHeading = $rootScope.magneticHeading;
        //$scope.spot.trueHeading = $rootScope.trueHeading;
        $scope.spot.direction = destinationBearing;
        $scope.spot.direction2 = $rootScope.currentHeading;
        var diff = destinationBearing - $rootScope.currentHeading;
        //$scope.spot.direction3 = diff;
        angular.element(document.querySelector('#arrow')).css({
            '-moz-transform': 'rotate(' + diff + 'deg)',
            '-webkit-transform': 'rotate(' + diff + 'deg)',
            '-o-transform': 'rotate(' + diff + 'deg)',
            '-ms-transform': 'rotate(' + diff + 'deg)'
        });
    }
})