angular.module('starter.services', [])

.factory('DistanceTools', function () {
    // convert to radians
    var rad = function (x) {
        return x * Math.PI / 180;
    };

    // convert to degrees
    var deg = function (x) {
        return x * 180 / Math.PI;
    };

    return {
        filterWithinRectangle: function (hotspots, currentCoords, kmRadius) {
            lon1 = currentCoords.longitude - kmRadius / Math.abs(Math.cos(rad(currentCoords.latitude)) * 69);
            lon2 = currentCoords.longitude + kmRadius / Math.abs(Math.cos(rad(currentCoords.latitude)) * 69);
            lat1 = currentCoords.latitude - (kmRadius / 69);
            lat2 = currentCoords.latitude + (kmRadius / 69);

            return hotspots.filter(function (spot) {
                return spot.latitude >= lat1 && spot.latitude <= lat2 && spot.longitude >= lon1 && spot.longitude <= lon2;
            });
        },
        getDistance: function (currentLat, currentLng, hotspotLat, hotspotLng) {
            var R = 6378137; // Earths mean radius in meters
            var dLat = rad(hotspotLat - currentLat);
            var dLong = rad(hotspotLng - currentLng);
            var a = Math.sin(dLat / 2) * Math.sin(dLat / 2) +
              Math.cos(rad(currentLat)) * Math.cos(rad(hotspotLat)) *
              Math.sin(dLong / 2) * Math.sin(dLong / 2);
            var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
            var d = R * c; // distance in meters
            return d;
        },
        getBearing: function (lat1,lng1,lat2,lng2) {
            var dLon = rad(lng2 - lng1);
            var y = Math.sin(dLon) * Math.cos(rad(lat2));
            var x = Math.cos(rad(lat1)) * Math.sin(rad(lat2)) - Math.sin(rad(lat1)) * Math.cos(rad(lat2)) * Math.cos(dLon);
            var bearing = deg(Math.atan2(y, x));
            return ((bearing + 360) % 360);
        },
        formatDistance: function (distance) {
            if (distance >= 1000) {
                return (distance / 1000).toFixed(2) + 'km';
            } else {
                if (distance < 20)
                    return '<20m';
                return Math.round(distance) + 'm';
            }
        }
    }
})

.factory('NetworkTools', function ($cordovaNetwork) {
    return {
        isOnline: function () {
            var online = true;
            try {
                online = $cordovaNetwork.isOnline();
            } catch (err) {
                console.log(err.message);
            }
            return online;
        }
    }
})

.factory('DateTimeTools', function () {
    return {
        getTimeDifferenceSeconds: function (date1) {
            var date2 = Date.now();
            if (date2 < date1) {
                date2.setDate(date2.getDate() + 1);
            }

            var diff = date2 - date1;
            return diff/1000;
        }
    }
})

.factory('LocationTools', function ($cordovaGeolocation, $cordovaDeviceOrientation, $rootScope) {
    watchGeolocation = null;
    watchCompass = null;

    return {
        startWatch: function () {
            console.log('LocationTools - start watch');       
            watchGeolocation = $cordovaGeolocation.watchPosition({ maximumAge: 0, timeout: 15000, enableHighAccuracy: true });
            watchGeolocation.then(
              null,
              function (err) {
                  console.log('LocationTools error (location) - ' + err.message);
                  $rootScope.$broadcast('CurrentPosition', { type: 'fail' });
              },
              function (position) {
                  console.log('LocationTools - got position');
                  $rootScope.currentPosition = position;
                  $rootScope.currentPositionTimestamp = Date.now();
                  $rootScope.$broadcast('CurrentPosition', { type: 'location' });
              });

            watchCompass = $cordovaDeviceOrientation.watchHeading({ frequency: 100 });
            watchCompass.then(
                null,
                function (err) {
                    console.log('LocationTools error (compass) - ' + err.message);
                },
                function (heading) {
                    console.log('LocationTools - got heading');
                    $rootScope.trueHeading = heading.trueHeading;
                    $rootScope.magneticHeading = heading.magneticHeading;
                    $rootScope.currentHeading =
                        heading.trueHeading ? heading.trueHeading >= 0 ? Math.round(heading.trueHeading) : Math.round(heading.magneticHeading)
                            : Math.round(heading.magneticHeading);
                    $rootScope.$broadcast('CurrentPosition', { type: 'compass' });
                });
        },
        stopWatch: function() {
            console.log('LocationTools - stop watch');
            if (watchGeolocation) $cordovaGeolocation.clearWatch(watchGeolocation.watchID);
            watchGeolocation = null;
            if (watchCompass) $cordovaDeviceOrientation.clearWatch(watchCompass.watchID);
            watchCompass = null;
        }
    }
})

.factory('EmailTools', function ($http) {
        return {
            $send: function (payload, callback, index) {
                var method = 'GET';
                var url = "https://api.sendgrid.com/api/mail.send.json?";
                $http({
                    method: method,
                    url: url + "api_user=mcafreewifi&api_key=mcapub666!" +
                        "&to=" + payload.to +
                        "&subject=" + payload.subject +
                        "&text=" + payload.text +
                        "&from=" + payload.from
                }).
                success(function (data, status) {
                    console.log(index);
                    callback('success', index);
                }).error(function (data, status) {
                    console.log(index);
                    callback('fail', index);
                });
            }
        }
})
    
.factory('UpdateTools', function ($http, $rootScope, $cordovaToast) {
    return {
        $updateNow: function (displayErrors, displayNoUpdates) {
            var newVersion;
            var method = 'GET';
            var versionUrl = "http://www.mca.org.mt/resources/lastupdate.txt?";
            var hotspotUrl = "http://www.mca.org.mt/resources/hotspots.txt?";
            $http({
                method: method,
                url: versionUrl
            }).
            success(function (data, status) {
                newVersion = data;
                console.log('Current Version - ' + $rootScope.$storage.currentVersion + ', New Version - ' + newVersion);
                if (parseInt(newVersion) > $rootScope.$storage.currentVersion) {
                    $http({
                        method: method,
                        url: hotspotUrl
                    }).
                    success(function (data, status) {
                        $rootScope.$storage.wifispots = data;
                        $rootScope.$storage.currentVersion = newVersion;
                        $rootScope.$storage.$apply();
                        console.log('NewVersion - ' + $rootScope.$storage.currentVersion);
                        console.log('Updated list');
                        $rootScope.$broadcast('UpdateHotspots');
                        $cordovaToast.showLongBottom('Wifi hotspot list updated - v' + newVersion);
                    }).error(function (data, status) {
                        if (displayErrors) {
                            $cordovaToast.showLongBottom('Unable to update hotspots at this time');
                        }
                    });
                } else {
                    if (displayNoUpdates)
                        $cordovaToast.showLongBottom('Wifi hotspot list is already up to date');
                }
            }).error(function (data, status) {
                if (displayErrors) {
                    $cordovaToast.showLongBottom('Unable to update hotspots at this time');
                }
            });
        }
    }
})

;